﻿// Copyright (c) 2021 Tudor Stanciu

using Microsoft.Extensions.Caching.Distributed;
using Netmash.Extensions.Caching.Extensions;
using System;
using System.Collections.Concurrent;
using System.Threading;
using System.Threading.Tasks;

namespace Netmash.Extensions.Caching.Services
{
    internal class CacheService : ICacheService
    {
        private readonly IDistributedCache _cache;
        private static readonly ConcurrentBag<string> _keys = new();

        const int expirationTimeFromNow = 24;

        public CacheService(IDistributedCache cache)
        {
            _cache = cache;
        }

        public async Task<T> GetAsync<T>(string key, CancellationToken token = default) where T : class
        {
            var result = await _cache.GetAsync<T>(key, token);
            return result;
        }

        public async Task SetAsync<T>(string key, T value, CancellationToken token = default) where T : class
        {
            _keys.Add(key);
            await _cache.SetAsync(key, value, new DistributedCacheEntryOptions() { AbsoluteExpirationRelativeToNow = TimeSpan.FromHours(expirationTimeFromNow) }, token);
        }

        public void Reset()
        {
            lock (_keys)
            {
                foreach (var key in _keys)
                    _cache.Remove(key);
                _keys.Clear();
            }
        }
    }
}
